#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2006-2007  Néstor Arocha Rodríguez, Inmaculada Luengo Merino

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Ventana principal"""


from qt import SIGNAL, QMessageBox, QErrorMessage, QFileDialog, qApp, QApplication, QToolButton, QIconSet, QWidget, QSize, QAction, QTextDrag, QString, QPopupMenu
from Driza.iuqt3.ui.vprincipal import vprincipal
from Driza.iuqt3.dconfig import DConfig
from Driza.iuqt3.dbuscar import DBuscar
from Driza.iuqt3.dfiltrado import DFiltrado
from Driza.iuqt3.dayuda import DAyuda
from Driza.iuqt3.vprincipal.grid import Grid
from Driza import listas
import logging
LOG = logging.getLogger("Driza.iuqt3.vprincipal.vprincipal")

class VPrincipal(vprincipal):
    """Ventana principal, hereda de vprincipal, que esta generado por el designer.
    Esta clase es padre de la mayoria de los dialogos, y es la que permite introducir los datos a el usuario"""

    def __arboltomenu(self, padre, arbol, diccionario):
        """Convierte un arbol a formato menu.  Recursiva"""
        for hijo in arbol.enlaces:
            nuevarama = QPopupMenu(self)
            padre.insertItem(QString(str(hijo.contenido)), nuevarama)
            diccionario[str(hijo.contenido)] = {"objeto":nuevarama}
            self.__arboltomenu(nuevarama, hijo, diccionario[str(hijo.contenido)])

    def __listaetiquetasmenu(self, diccionario, etiquetasanteriores):
        """Devuelve una lista con parejas qpopup, listatags"""
        lista = []
        for (indice, valor) in diccionario.items():
            if isinstance(valor, dict):
                nuevasetiquetas = etiquetasanteriores[:]
                nuevasetiquetas.append(indice)
                for resultado in self.__listaetiquetasmenu(valor, nuevasetiquetas):
                    lista.append(resultado)
            else:
                lista.append([valor, etiquetasanteriores])
        return lista



    def __init__(self, parent, config, portero, idu, gestorproyectos, gestortemas, gestoroperaciones): 
        """Inicializador de la ventana principal"""
        vprincipal.__init__(self, None)
        #VARIABLES PUBLICAS
        self.parent = parent
        self.__config = config
        self.__gestortemas = gestortemas
        self.__idu = idu
        #Bloque de inicialización de dialogos
        self.__dbuscar = DBuscar(self, "busqueda")
        self.__dconfig = DConfig(config, self)
        self.__dfiltro = DFiltrado(self, self.__idu)
        self.__dayuda = DAyuda(self)
        self.__portero = portero
        self.__gestorproyectos = gestorproyectos
        self.grid = Grid(self, self.__idu, self.__portero)
        self.widgetStack1.addWidget(self.grid)
        self.widgetStack1.raiseWidget(self.grid)

        # Funciones de inicializacion de componentes
        self.__inittoolbar()
        self.showundoredo()

        self.setIcon(self.__gestortemas.iconoprograma())
        self.__diccionarioarbolmenu = {}
        self.__diccionarioacciones = {}
        self.__colectorfunciones = []
        listalistasetiquetas = [operacion.etiquetas for operacion in gestoroperaciones.values()]
        from Driza import categorias
        arbol = categorias.categorias2arbol("Raiz", listalistasetiquetas)
        self.__arboltomenu(self.Analizar, arbol, self.__diccionarioarbolmenu)
        listaelementosmenu =  self.__listaetiquetasmenu(self.__diccionarioarbolmenu, [])
        from sets import Set
        for operacion in gestoroperaciones.values():
            for elementofinal in listaelementosmenu:
                if Set(operacion.etiquetas) == Set(elementofinal[1]):
                    accion = QAction(self, str(operacion.nombre))
                    accion.setText(str(operacion.nombre))
                    accion.addTo(elementofinal[0])
                    self.__diccionarioacciones[operacion.nombre] = accion
        self.__atajos()


    #FUNCIONES PUBLICAS
    
    def abrirfichero(self, parent = None, filename = None):
        """ Lanza el diálogo de apertura de fichero """
        from Driza.excepciones import FicheroNoExisteException, FicheroErroneoException, FicheroTipoDesconocidoException
        if not parent: 
            parent = self
        if not self.__preguntarproyectomodificado(): 
            return
        if not filename:
            filter = ""
            for fmt in listas.extensiones_fichero():
                filter = filter + "%s files (*.%s);;" % (fmt, fmt.lower())
            fn = QFileDialog.getOpenFileName(QString.null, filter, self, None, "Dialogo abrir fichero", "")
            filename = str(fn)
        if filename:
            try:
                self.__gestorproyectos.cargar(filename)
            except FicheroErroneoException:
                QMessageBox.warning(parent, u'Atención', 'El fichero no ha podido ser leido')
            except FicheroTipoDesconocidoException:
                QMessageBox.warning(parent, u'Atención', 'El fichero no ha podido ser leido')
            except AttributeError:
                QErrorMessage(parent, "Error").message(u"Parece que ha intentado cargar un fichero de una versión anterior. Lo siento")
                LOG.exception("excepcion capturada")
            else:
                self.myUpdate()
                self.grid.myUpdate()
                return True
        return False

    def myUpdate(self):
        """Acciones de actualizacion"""
        from Driza import VERSION
        if self.__gestorproyectos.fichero:
            nfichero = self.__gestorproyectos.fichero
        else:
            nfichero = "Nuevo fichero"
        self.setCaption("Driza " + VERSION + " - " + nfichero)

    def showEvent(self, event):
        self.myUpdate()
        vprincipal.showEvent(self, event)

    def closeEvent(self, event):
        """Acciones que se realizaran cuando el usuario cierre la ventana. Guarda el estado de la configuración,
        incluyendo los ficheros recientemente abiertos"""
        if not self.__preguntarproyectomodificado(): 
            return
        QWidget.closeEvent(self, event)
        if not self.parent.vsalida.isVisible():
            self.__config.save()
            qApp.exit(0)
        else:
            self.hide()


    
    def showundoredo(self):
        """Determina que botones estan activados y cuales no del submenu de edición"""
        if self.__portero.puedoundo():
            self.editUndoAction.setEnabled(True)
        else:
            self.editUndoAction.setEnabled(False)

        if self.__portero.puedoredo():
            self.editRedoAction.setEnabled(True)
        else:
            self.editRedoAction.setEnabled(False)

    #FUNCIONES PRIVADAS

    def __importar(self):
        """Importa los datos de un fichero"""
        if not self.__preguntarproyectomodificado(): return
        self.parent.dimportartexto.show()

    def __preguntarproyectomodificado(self):
        """Pregunta en caso de que haya sido modificado el proyecto si desea ser guardado"""
        if not self.__idu.original():
            returncode = QMessageBox.information(self, 'Atencion:', 'El proyecto actual ha sido modificado, desea guardarlo?', 'Guardarlo', 'No guardarlo', 'Volver', 0, 1)
            if returncode == 0:
                self.__guardar()
            elif returncode == 2:
                return False
        return True


    def __dacerca_de(self):
        """Mensaje acerca de"""
        from Driza import VERSION
        separador = "\n----------------------------------\n"
        betatesters = u"Carlos Mestre Gonzalez\n Luis de Bethencourt Guimerá"
        iconos = "Iconos del Tango Desktop Project: http://tango.freedesktop.org/"
        ristra = u"Driza %s es una interfaz estadística\n (C) Néstor Arocha Rodríguez - Tutorizado por Inmaculada luengo Merino\n Distribuido bajo licencia GPL" +\
                separador + "Betatesters:\n" + betatesters + separador +\
                u" Mantenedor del paquete .deb:\n Luis de Bethencourt Guimerá"+ separador + iconos
        QMessageBox.about(self, "Acerca de Driza", ristra % (VERSION))
    
    def __undo(self):
        """Deshace el último cambio"""
        self.__portero.undo()
        self.grid.myUpdate()
        self.showundoredo()
    
    def __redo(self):
        """Rehace el último cambio"""
        self.__portero.redo()
        self.grid.myUpdate()
        self.showundoredo()

    def __inittoolbar(self):
        """Inicializa la toolbar"""
        self.boton1 = QToolButton(self.Toolbar)
        self.boton1.setTextLabel("ABRIR")
        iconos = QIconSet()
        iconos.setIconSize(QIconSet.Small, QSize(100, 100))
        iconos.setPixmap(self.__gestortemas.iconoabrir(), QIconSet.Small)
#        iconos.setPixmap("images/flechaabajo.png", QIconSet.Automatic, QIconSet.Normal, QIconSet.Off)
        self.boton1.setOn(True)
        self.boton1.setIconSet(iconos)
        self.boton2 = QToolButton(self.Toolbar)
        self.boton2.setTextLabel("GUARDAR")
        iconos = QIconSet()
        iconos.setIconSize(QIconSet.Small, QSize(100, 100))
        iconos.setPixmap(self.__gestortemas.iconoguardar(), QIconSet.Small)
        self.boton2.setOn(True)
        self.boton2.setIconSet(iconos)
        self.boton3 = QToolButton(self.Toolbar)
        self.boton3.setTextLabel("Ventana Salida")
        iconos = QIconSet()
        iconos.setIconSize(QIconSet.Small, QSize(100, 100))
        iconos.setPixmap(self.__gestortemas.icononuevaventana(), QIconSet.Small)
        self.boton3.setOn(True)
        self.boton3.setIconSet(iconos)
        self.boton4 = QToolButton(self.Toolbar)
        self.boton4.setTextLabel("Mostrar etiquetas")
        self.boton4.setToggleButton(True)
        iconos = QIconSet()
        iconos.setIconSize(QIconSet.Small, QSize(100, 100))
        iconos.setPixmap(self.__gestortemas.iconoetiquetas(), QIconSet.Small)
        self.boton4.setIconSet(iconos)

    def conexiones(self):
        """Funcion llamada en el constructor que almacena todas las conexiones accion-funcion"""
        self.connect(self.archivoSalirAction, SIGNAL("activated()"), self.__salirprograma)
        self.connect(self.fileNewAction, SIGNAL("activated()"), self.__nuevo)
        self.connect(self.fileOpenAction, SIGNAL("activated()"), self.abrirfichero)
        self.connect(self.fileSaveAsAction, SIGNAL("activated()"), self.__guardarcomo)
        self.connect(self.fileSaveAction, SIGNAL("activated()"), self.__guardar)
        self.connect(self.archivoImportarAction, SIGNAL("activated()"), self.__importar)
        self.connect(self.archivoConfiguracinAction, SIGNAL("activated()"), self.__dconfig.show)
        self.connect(self.datosFiltrarAction, SIGNAL("activated()"), self.__dfiltro.show)
        self.connect(self.modificarCrear_nuevas_variablesAction, SIGNAL("activated()"), self.parent.dcrevar.show)
        self.connect(self.editFindAction, SIGNAL("activated()"), self.__dbuscar.show)
        self.connect(self.editCopyAction, SIGNAL("activated()"), self.__copy)
        self.connect(self.editCutAction, SIGNAL("activated()"), self.__cut)
        self.connect(self.editPasteAction, SIGNAL("activated()"), self.__paste)
        self.connect(self.editUndoAction, SIGNAL("activated()"), self.__undo)
        self.connect(self.editRedoAction, SIGNAL("activated()"), self.__redo)
        self.connect(self.edicinBorrarAction, SIGNAL("activated()"), self.__borrar)
        self.connect(self.edicinInsertarAction, SIGNAL("activated()"), self.__insertar)
        self.connect(self.acerca_deAcerca_deAction, SIGNAL("activated()"), self.__dacerca_de)
        self.connect(self.boton1, SIGNAL("clicked()"), self.abrirfichero)
        self.connect(self.boton2, SIGNAL("clicked()"), self.__guardar)
        self.connect(self.boton3, SIGNAL("clicked()"), self.parent.vsalida.show)
        self.connect(self.boton4, SIGNAL("clicked()"), self.__toggleetiquetas)
        self.connect(self.ayudaAyudaAction, SIGNAL("activated()"), self.__mostrardayuda)
        self.connect(self.PopupMenuEditor_3, SIGNAL("aboutToShow()"), self.__actualizarrecientes)
        for (key, valor) in self.__diccionarioacciones.items():
            self.__colectorfunciones.append(lambda k=key: self.parent.doperaciones.mostrar(k))
            self.connect(valor, SIGNAL("activated()"), self.__colectorfunciones[-1])

    def __atajos(self):
        """Establece los atajos"""
        self.ayudaAyudaAction.setAccel("F1")
        self.acciontab1 = QAction(self, "tabcasos")
        self.connect(self.acciontab1, SIGNAL("activated()"), self.grid.showtabcasos)
        self.acciontab1.setAccel("F2")
        self.acciontab2 = QAction(self, "tabvars")
        self.connect(self.acciontab2, SIGNAL("activated()"), self.grid.showtabvariables)
        self.acciontab2.setAccel("F3")
        self.__diccionarioacciones["Descriptivo"].setAccel("F4")
        self.boton3.setAccel("F5") #Ventana salida
        self.archivoConfiguracinAction.setAccel("F10")
        self.acerca_deAcerca_deAction.setAccel("F12")
        self.archivoSalirAction.setAccel("Ctrl+Q")

    def __copy(self):
        """Funcion que copia y borra la seleccion"""
        try:
            self.__copiar()
        except AssertionError:
            QMessageBox.warning(self, u'Atención', 'Las operaciones de copiado, cortado y pegado no han sido implementadas')
        else:
            self.grid.removeselection() 

   #http://www.google.com/codesearch?hl=en&q=+%22qtable%22+%22cut%22+%22paste%22+show:z9otKZeV6U8:R6dK3Cx-dYg:Gh37-3Ie27E&sa=N&cd=48&ct=rc&cs_p=http://mde.abo.fi/confluence/download/attachments/1011/coral-0.9.1.tar.gz&cs_f=coral-0.9.1/coral/modeler/property/propertyeditor.py#a0 
    def __cut(self):
        """Copia, borra la seleccion y su contenido"""
        from Driza.excepciones import SeleccionIncorrectaException
        try:
            self.grid.verificarSeleccionRegistros()
            self.__portero.guardarestado()
            self.__copiar()
        except SeleccionIncorrectaException:
            QMessageBox.warning(self, u'Atención', 'Solo se permite cortar registros')
        except AssertionError:
            QMessageBox.warning(self, u'Atención', 'Las operaciones de copiado, cortado y pegado no han sido implementadas')
        else:
            self.grid.removeselection(borrardatos=True)
            self.grid.myUpdate()


    def __paste(self):
        """Funcion que envuelve a __pegar recogiendo sus excepciones"""
        #try:
        if 1:
            self.__pegar()
        #except:
        #    QErrorMessage(self, "Error").message(u"Error desconocido")
        #    LOG.exception("excepcion capturada en el pegado")

    def __pegar(self):
        """Pega lo que haya sido copiado"""
        clipboard = QApplication.clipboard()
        registro = clipboard.text().latin1()
        lista = eval(registro)
        posrow = self.grid.table1.currentRow()
        poscol = self.grid.table1.currentColumn()
        if len(lista[0]) == self.__idu.numVar():
            poscol = 0 #Copia de registro completo
        else:
            #No permitimos pegar si el pegado implica variables nuevas
            assert(poscol + len(lista[0]) <= self.__idu.numVar())
        self.__portero.guardarestado() #Guardamos el estado

        #Creamos nuevos registros hasta la posicion del cursor
        if posrow > self.__idu.numReg():
            for _ in range(self.__idu.numReg(), posrow):
                self.__idu.appReg()
        #Hacemos el pegado efectivo
        i = posrow
        for registro in lista:
            if i >= self.__idu.numReg(): 
                self.__idu.appReg()
            j = poscol
            for campo in registro:
                if campo != None:
                    self.__idu[i][j] = campo
                j += 1
            i += 1
        self.grid.myUpdate()

    def __salirprograma(self):
        """Funcion de salida del programa. Guarda la configuracion"""
        if not self.__preguntarproyectomodificado(): 
            return
        self.__config.save()
        qApp.exit(0)


    def __nuevo(self):
        """Accion realizada cuando el usuario clickea en Nuevo"""
        if not self.__preguntarproyectomodificado(): 
            return
        self.__idu.borrartodo()
        self.__gestorproyectos.fichero = None
        self.myUpdate()
        self.grid.myUpdate()


    
    def __guardar(self):
        """Funcion de guardado del fichero. Si el fichero no es conocido se llama a guardarcomo"""
        if self.__gestorproyectos.fichero:
            self.__gestorproyectos.guardar(sobreescribir = True)
            self.__idu.setFlagoriginal()
        else:
            self.__guardarcomo()

    def __guardarcomo(self):
        """Abre un diálogo pidiendo el nombre de archivo y guarda en dicho archivo"""
        filter = ""
        for fmt in listas.extensiones_fichero():
            filter = filter+ "%s files (*.%s);;" % (fmt, fmt.lower())
        fn = QFileDialog.getSaveFileName(QString.null, filter, self)
        filename = str(fn)
        if filename:
            from Driza.excepciones import FicheroExisteException, FicheroTipoDesconocidoException
            import re
            extension = re.compile('.*\..*')
            if not extension.match(filename):
                filename += ".driza"
            try:
                self.__gestorproyectos.guardar(filename)
            except FicheroExisteException, fichero:
                returncode = QMessageBox.information(self, 'Atencion:', 'El fichero' + fichero.fichero + ' ya existe' , 'Sobreescribir', 'Otro nombre', 'Cancelar', 0, 1)
                if returncode == 0:
                    self.__gestorproyectos.guardar(filename, True)
                    self.__idu.setFlagoriginal()
                    self.myUpdate()
                elif returncode == 1:
                    self.__guardarcomo()
            except FicheroTipoDesconocidoException:
                QMessageBox.warning(self, u'Atención', u'La extensión del fichero es incorrecta.\nPruebe con otra extensión')
                self.__gestorproyectos.fichero = None
            else:
                self.__idu.setFlagoriginal()
                self.myUpdate()

                    
    def __copiar(self):
        """Esta funcion pega el contenido del clipboard en la tabla actual. Es llamada por copy"""
        tablaactual = self.grid.currentPageIndex()
        assert(tablaactual == 0) #TODO Cambiar por una excepcion más concreta
        lista = self.grid.listaseleccion()
        cadena = repr(lista)
        clipboard = QApplication.clipboard()
        clipboard.setData(QTextDrag(cadena))
            

    def __mostrardayuda(self):
        self.__dayuda.mostrar("vprincipal")

    def __borrar(self):
        self.grid.removeselection(borrardatos = True)
        self.grid.myUpdate()

    def __insertar(self):
        #try:
        if 1:
            self.grid.insertarcaso()
        #except:
        #    QMessageBox.warning(self, u'Atención', u'Error en la inserción')
        #    LOG.exception("excepcion capturada al insertar")

    def __toggleetiquetas(self):
        LOG.debug("Cambiado el modo etiqueta")
        self.grid.modoetiqueta = not self.grid.modoetiqueta
        self.grid.myUpdate()

    def __actualizarrecientes(self):
        self.PopupMenuEditor_3.clear()
        for i in range(len(self.__config.configuracion["lfichero"]) - 1):
            self.__colectorfunciones.append(lambda nombre=self.__config.configuracion["lfichero"][i]: self.abrirfichero(self, nombre))
            accion = QAction(self, self.__config.configuracion["lfichero"][i])
            accion.setText(self.__config.configuracion["lfichero"][i])
            accion.addTo(self.PopupMenuEditor_3)
            self.connect(accion, SIGNAL("activated()"), self.__colectorfunciones[-1])

